% imgBinStruct

%============================ EXAMPLE INPUT ==============================%
% %%%%%%%%%%%%%%% Inputs
% images = 'D:\R2016a\Project files\Matlab videos\16.03.17 oldStup\Au8oh8cooh_CA_60s_off_210s_0mV_210s_-600mV_stk';
% %%%%%%%%%%%%%%%
% % EXAMPLE RESULT SAVE NAME: % 16_03_17_CA_bin4_20s_entropy_images
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%% Binning Start %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%% Inputs_start
% bin_length = 4;
% image_descriptor = '16.03.17 oldStup__Au8oh8cooh_CA_60s_off_210s_0mV_210s_-600mV_stk';
%%%%%%%%%%%%% Inputs_end
%=========================================================================%

% ~~~~ REQUIRED ~~~~ 
% binSize :- is the length/width/hight of the bin.
% images :- either a folder of images or a single image in matrix format.
% ~~~~ OPTIONAL ~~~~ 
% imageDescription :- string decription that will apear in the resultant data struct.
%                   - Defaults to the filename if non provided.
% ...FileNum :- 0 defaults to start/end of file.
% startFileNum :- inclusive starting file index.
% endFileNum :- inclusive end file index.

function imageStorage = imgBinStruct(images, binSize, imageDescription, startFileNum, endFileNum)

% -------- INPUT PROCESSING -------- %
decriptionString = 'binned image';
usingImageFolder = true;
requireExtraction = true;

if nargin >= 4 % if exist('startFileNum','var')
    fileNumStart = startFileNum;
else
    fileNumStart = 0;
end
if nargin >= 5 % if exist('endFileNum','var')
    fileNumEnd = endFileNum;
else
    fileNumEnd = 0;
end

if isnumeric(images)
    usingImageFolder = false;
    requireExtraction = false;
elseif ~isdir(images)
    usingImageFolder = false;
end
bin_length = binSize;
filePattern = 'null';
theFilesList = 'null';
numb_of_files = 1;
pre_img01 = 0;
% Image parameter extraction
if requireExtraction
    if nargin >= 3
        decriptionString = imageDescription;
    else
        decriptionString = images;
    end
    
    if usingImageFolder
        filePattern = fullfile(images, '*.tif');
        theFilesList = dir(filePattern);
        if fileNumStart == 0
            fileNumStart = 1;
        end
        if fileNumEnd == 0
            fileNumEnd = length(theFilesList(:));
        end
        numb_of_files = (fileNumEnd - fileNumStart)+1;
        pre_img01 = imread(fullfile(images,theFilesList(fileNumStart).name));
    else
        fileNumStart = 1;
        fileNumEnd = 1;
        pre_img01 = imread(images);
    end
else
    fileNumStart = 1;
    fileNumEnd = 1;
    pre_img01 = images;
end

if size(pre_img01,3)==3
    pre_img01 = rgb2gray(pre_img01);
end

[st_rows,st_cols] = size(pre_img01);
st_size = st_rows * st_cols;

%%%%%%%%%%%%% Image Data Container
imageStorage = struct;
imageStorage.name = decriptionString;
imageStorage.count = numb_of_files;
imageStorage.binned_container = cell(numb_of_files,1);
imageStorage.original = struct;
imageStorage.original.rows = st_rows;
imageStorage.original.cols = st_cols;
imageStorage.original.size = st_size;
imageStorage.binned = struct;

f1 = 1;
%%%%%%%%%%%%% Data transfer +/- binning
if bin_length < 2
    
    bin_length = 1;
    binned_rows = st_rows;
    binned_cols = st_cols;
    binned_size = binned_rows*binned_cols;
%     binned_img = zeros(binned_rows, binned_cols, 'uint8');
    
    imageStorage.binned.rows = binned_rows;
    imageStorage.binned.cols = binned_cols;
    imageStorage.binned.size = binned_size;
    imageStorage.bin_length = bin_length;
    if numb_of_files > 1
        for fr = fileNumStart:fileNumEnd
            if (rem(fr,500)==0)
                fprintf('processing file number %i of %i.\n', fr,numb_of_files);
            end
            imageStorage.binned_container{f1} = imread(fullfile(images,theFilesList(fr).name));
            f1 = f1 + 1;
        end
    else
       imageStorage.binned_container{1} = pre_img01; 
    end
        
    
else
    
    % bin_length modification if the entered in value does not factor with both rows and columns.
    if rem(st_rows,bin_length) ~= 0 || rem(st_rows,bin_length) ~= 0
        if bin_length < 8
            while rem(st_rows,bin_length) ~= 0 || rem(st_rows,bin_length) ~= 0
                bin_length = bin_length + 1;
            end
        else
            while rem(st_rows,bin_length) ~= 0 || rem(st_rows,bin_length) ~= 0
                bin_length = bin_length - 1;
            end
        end
    end

    binned_rows = st_rows/bin_length;
    binned_cols = st_cols/bin_length;
    binned_size = binned_rows*binned_cols;
    binned_img = zeros(binned_rows, binned_cols, 'uint16');

    bin_pixels = bin_length*bin_length;
    bin_factor = 1/bin_length;

    imageStorage.binned.rows = binned_rows;
    imageStorage.binned.cols = binned_cols;
    imageStorage.binned.size = binned_size;
    imageStorage.bin_length = bin_length;


    %%%% method_1 part_1 --- produce index map ------
    binned_indexer = zeros(st_rows,st_cols,'uint32');
    for c = 1:st_cols
        c2 = floor((c-1)*bin_factor);
        for r = 1:st_rows
            r2 = ceil(r*bin_factor);
            binned_indexer(r,c) = (binned_rows*c2) + r2;
        end
    end
    %%%% method_1 part_1 END

    for fr = fileNumStart:fileNumEnd 
        if (rem(fr,500)==0)
            fprintf('processing file number %i of %i.\n', fr,numb_of_files);
        end
        if numb_of_files > 1
            imgX = imread(fullfile(images,theFilesList(fr).name));
        else
            imgX = pre_img01; 
        end
        %%%% method_1 part_2
        % ---- Idealy should vectorise the loop below? ----
        for i = 1:st_size
            i2 = binned_indexer(i);
            binned_img(i2) = binned_img(i2) + uint16(imgX(i));
        end
        %%%% method_1 part_2 END
        %%%%% re-adjust intensity/averageing divide
        imageStorage.binned_container{f1} = uint8(binned_img/bin_pixels);
        f1 = f1 + 1;
        binned_img = zeros(binned_rows, binned_cols, 'uint16');
        %%%%% re-adjust END
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Binning End %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%